#ifndef __Db_hxx__
#define __Db_hxx__

#ifdef __cplusplus
#define DbReference DbCCReference
#define DbString    DbCCString
#define DbVector    DbCCVector
#define DbMatrix    DbCCMatrix
#endif

#include <Db.h>

#ifdef __cplusplus
#undef DbMatrix
#undef DbVector
#undef DbString
#undef DbReference
#endif

//
//---------------------------------------------------------
//

#ifdef __cplusplus
class DbChar
{
  public :

  DbChar ();
  DbChar (char value);
  ~DbChar ();

  DbChar& operator = (char value);
  operator char ();
  operator char () const;

  int operator == (const DbChar& other);
  int operator != (const DbChar& other);
  static DbType _type ();
  static DbChar& _null ();

  private :

  char _value;
};

//
//---------------------------------------------------------
//

class DbShort
{
  public :

  DbShort ();
  DbShort (short value);
  ~DbShort ();

  DbShort& operator = (short value);
  operator short ();
  operator short () const;

  int operator == (const DbShort& other);
  int operator != (const DbShort& other);
  static DbType _type ();
  static DbShort& _null ();

  private :

  short _value;
};

//
//---------------------------------------------------------
//

class DbInt
{
  public :

  DbInt ();
  DbInt (int value);
  ~DbInt ();

  DbInt& operator = (int value);
  operator int ();
  operator int () const;

  int operator == (const DbInt& other);
  int operator != (const DbInt& other);
  static DbType _type ();
  static DbInt& _null ();

  private :

  int _value;
};

//
//---------------------------------------------------------
//

class DbHexa
{
  public :

  DbHexa ();
  DbHexa (int value);
  ~DbHexa ();

  DbHexa& operator = (int value);
  operator int ();
  operator int () const;

  int operator == (const DbHexa& other);
  int operator != (const DbHexa& other);
  static DbType _type ();
  static DbHexa& _null ();

  private :

  int _value;
};

//
//---------------------------------------------------------
//

class DbFloat
{
  public :

  DbFloat ();
  DbFloat (float value);
  ~DbFloat ();

  DbFloat& operator = (float value);
  operator float ();
  operator float () const;

  int operator == (const DbFloat& other);
  int operator != (const DbFloat& other);
  static DbType _type ();
  static DbFloat& _null ();

  private :

  float _value;
};

//
//---------------------------------------------------------
//

class DbDouble
{
  public :

  DbDouble ();
  DbDouble (double value);
  ~DbDouble ();

  DbDouble& operator = (double value);
  operator double ();
  operator double () const;

  int operator == (const DbDouble& other);
  int operator != (const DbDouble& other);
  static DbType _type ();
  static DbDouble& _null ();

  private :

  double _value;
};

//
//---------------------------------------------------------
//

class DbString : public DbCCString
{
  public :
  DbString ();
  ~DbString ();

  DbString& operator = (const char* text);
  DbString& operator = (const DbString& other);
  operator char* () const;

  DbString& operator += (const char* text);
  DbString& operator += (const DbString& other);

  DbString& operator -= (const char* text);
  DbString& operator -= (const DbString& other);

  void replace (const char* original_text, const char* new_text);
  void replace_all (const char* original_text, const char* new_text);

  int operator == (const DbString& other);
  int operator != (const DbString& other);
  static DbType _type ();
  static DbString& _null ();
};

//
//---------------------------------------------------------
//

template <class T> class DbReference : public DbCCReference
{
  public :
  DbReference ();
  DbReference (const char* name);
  ~DbReference ();

  DbReference& operator = (const char* name);
  DbReference& operator = (const DbReference<T>& other);
  operator char* () const;
  void update ();

  int operator == (const DbReference<T>& other);
  int operator != (const DbReference<T>& other);
  static DbType _type ();
  static DbReference<T>& _null ();
};

//
//---------------------------------------------------------
//

template <class T> class DbVector : public DbCCVector
{
  public:

  DbVector ();
  ~DbVector ();
  int size () const;
  T& operator [] (int i);

  int operator == (const DbVector<T>& other);
  int operator != (const DbVector<T>& other);
  static DbType _type ();
  static DbVector<T>& _null ();
};

//
//---------------------------------------------------------
//

template <class T> class DbMatrix : public DbCCMatrix
{
  public:

  DbMatrix (int rows, int columns);
  ~DbMatrix ();
  int rows () const;
  int columns () const;
  T& operator () (int row, int column);

  int operator == (const DbMatrix<T>& other);
  int operator != (const DbMatrix<T>& other);
  static DbType _type ();
  static DbMatrix<T>& _null ();
};

//
//---------------------------------------------------------
//

template <class T> class DbPtr
{
  public :

  static void lock (const char* name);
  static void unlock (const char* name);

  public :

  DbPtr ();
  DbPtr (const char* name);
  DbPtr (DbReference<T>& reference);
  ~DbPtr ();

  void load (const char* name);
  void load_by_tag (const char* name, const char* tag);
  void load_by_date (const char* name, const char* date);
  void load (DbReference<T>& reference);

  int is_valid ();
  T& operator * ();
  T* operator -> ();
  void commit ();

  private:
  DbObject _object;
};

#include <Db.icc>
#endif

#endif
