/*---------------------------------------------------------------------------*/
/*        Copyright (c) 1996 LAL Orsay, UPS-IN2P3-CNRS (France).             */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/* 1. Redistributions of source code must retain the above copyright         */
/*    notice, this list of conditions and the following disclaimer.          */
/* 2. Redistributions in binary form must reproduce the above copyright      */
/*    notice, this list of conditions and the following disclaimer in the    */
/*    documentation and/or other materials provided with the distribution.   */
/* 3. All advertising materials mentioning features or use of this software  */
/*    must display the following acknowledgement:                            */
/*      This product includes software developed by the Computer Application */
/*      Development Group at LAL Orsay (Laboratoire de l'Accelerateur        */
/*      Linaire - UPS-IN2P3-CNRS).                                           */
/* 4. Neither the name of the Institute nor of the Laboratory may be used    */
/*    to endorse or promote products derived from this software without      */
/*    specific prior written permission.                                     */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE LAL AND CONTRIBUTORS ``AS IS'' AND       */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE     */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR        */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE LAL OR CONTRIBUTORS BE      */
/* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR       */
/* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF      */
/* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS  */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN   */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)   */
/* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF    */
/* THE POSSIBILITY OF SUCH DAMAGE.                                           */
/*---------------------------------------------------------------------------*/

/*- Header -------------------------------------------------------*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <Db.h>
#include <CmMessage.h>
/*----------------------------------------------------------------*/

/*----------------------------------------------------------------*/
int vprintf (const char *format, va_list ap);
/*----------------------------------------------------------------*/

/*----------------------------------------------------------------*/
static DbPrinter  Printer            = NULL;
static DbError    Error              = DbErrorOk;
static char       ErrorText[2048]    = { 0 };
static char*      ErrorName[]        = {
  "DbErrorOk",
  "DbErrorNoCm",
  "DbErrorNoHandler",
  "DbErrorConnectionLost",
  "DbErrorCmError",
  "DbErrorBadType",
  "DbErrorNotStructure",
  "DbErrorBadArgument",
  "DbErrorTypeAlreadyDefined",
  "DbErrorNoServer",
  "DbErrorUnknownStructure",
  "DbErrorBadSyntax",
  "DbErrorBadFieldSyntax",
  "DbErrorBadDimension",
  "DbErrorCannotOpenFile",
  "DbErrorCannotCreateFile",
  "DbErrorUnknownObject",
  "DbErrorUnloadedStructure",
  "DbErrorObjectAlreadyDefined",
  "DbErrorNotYetImplemented",
  "DbErrorTypeMismatch",
  "DbErrorCommitDenied",
  "DbErrorReadAccessDenied",
  "DbErrorWriteAccessDenied",
  "DbErrorCVSNotAvailable",
  "DbErrorCVSRepositoryUndefined",
  "DbErrorCVSModuleUnavailable",
  "DbErrorCVSUnloadedObject",
  "DbErrorCVS"
};
/*----------------------------------------------------------------*/

/*----------------------------------------------------------------*/
int DbPrint (char* format, ...)
/*----------------------------------------------------------------*/
{
  va_list args;

  if (Printer == NULL)
    {
      Printer = (DbPrinter) vprintf;
    }

  va_start (args, format);
  
  Printer (format, args);
      
  va_end (args);

  return (0);
}

/*----------------------------------------------------------------*/
DbPrinter DbInstallPrinter (DbPrinter printer)
/*----------------------------------------------------------------*/
{
  DbPrinter old = Printer;

  DbStartup ();

  Printer = printer;

  return (old);
}

/*-----------------------------------------------------------*/
void DbClearErrorInfo ()
/*-----------------------------------------------------------*/
{
  Error = DbErrorOk;
  ErrorText[0] = 0;
}

/*-----------------------------------------------------------*/
DbError DbGetError ()
/*-----------------------------------------------------------*/
{
  return (Error);
}

/*-----------------------------------------------------------*/
char* DbGetErrorName (DbError error)
/*-----------------------------------------------------------*/
{
  return (ErrorName[error]);
}

/*-----------------------------------------------------------*/
char* DbGetErrorInfo ()
/*-----------------------------------------------------------*/
{
  return (ErrorText);
}

/*-----------------------------------------------------------*/
void DbPrintError ()
/*-----------------------------------------------------------*/
{
  if (Error == DbErrorOk)
    {
      DbPrint ("Db> everything is ok\n");
    }
  else
    {
      DbPrint ("Db> %s : %s\n", ErrorName[Error], ErrorText);
    }
}

/*-----------------------------------------------------------*/
void DbSetErrorInfo (DbError error, char* format, ...)
/*-----------------------------------------------------------*/
{
  va_list args;
  static char text[256];

  if (format == NULL) return;

  va_start (args, format);
  vsprintf (text, format, args);
  va_end (args);

  Error = error;

  ErrorText[0] = 0;

  if (strlen (text) == 0) return;

  strcpy (ErrorText, text);

  if (DbDebug ())
    {
      DbPrint ("Db> %s\n", text);
    }
}

