#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Db.h>

DbType DbTHexaNew ();

/*---------------------------------------------------------------------------*/
unsigned int* DbHexaReferenceGetAddress (DbReference* reference)
/*---------------------------------------------------------------------------*/
{
  DbObject object;
  unsigned int* result;

  object = DbReferenceGetObject (reference);

  if (!DbTypeIsHexa (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaReferenceGetAddress>");
      return (NULL);
    }

  result = (unsigned int*) DbHexaObjectGetAddress (object);
  return (result);
}

/*---------------------------------------------------------------------------*/
unsigned int* DbHexaObjectLoad (char* name)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObject (name);
  if (DbGetError () != DbErrorOk) return (NULL);

  if (!DbTypeIsHexa (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaObjectLoad>");
      return (NULL);
    }

  return ((unsigned int*) DbHexaObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
unsigned int* DbHexaObjectLoadByTag (char* name, char* tag)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObjectByTag (name, tag);
  if (DbGetError () != DbErrorOk) return (NULL);

  if (!DbTypeIsHexa (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaObjectLoad>");
      return (NULL);
    }

  return ((unsigned int*) DbHexaObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
unsigned int* DbHexaObjectLoadByDate (char* name, char* date)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObjectByDate (name, date);
  if (DbGetError () != DbErrorOk) return (NULL);

  if (!DbTypeIsHexa (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaObjectLoad>");
      return (NULL);
    }

  return ((unsigned int*) DbHexaObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
DbVector* DbHexaVectorNew ()
/*---------------------------------------------------------------------------*/
{
  return (DbVectorNew (DbTHexaNew ()));
}

/*---------------------------------------------------------------------------*/
void DbHexaVectorInit (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  DbVectorInit (vector, DbTHexaNew ());
}

/*---------------------------------------------------------------------------*/
unsigned int* DbHexaVectorGetArray (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  unsigned int* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsHexa (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaVectorGetArray>");
      return (NULL);
    }

  result = (unsigned int*) DbVectorGetAddress (vector, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
unsigned int DbHexaVectorGetElement (DbVector* vector, int index)
/*---------------------------------------------------------------------------*/
{
  unsigned int* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsHexa (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaVectorGetElement>");
      return (0);
    }

  if (index >= DbVectorGetSize (vector))
    {
      DbVectorExtend (vector, index + 1);
    }

  result = (unsigned int*) DbVectorGetAddress (vector, index);

  return (*result);
}

/*---------------------------------------------------------------------------*/
void DbHexaVectorSetElement (DbVector* vector, 
                              int index, 
                              unsigned int value)
/*---------------------------------------------------------------------------*/
{
  unsigned int* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsHexa (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaVectorSetElement>");
      return;
    }

  if (index >= DbVectorGetSize (vector))
    {
      DbVectorExtend (vector, index + 1);
    }

  result = (unsigned int*) DbVectorGetAddress (vector, index);

  *result = value;
}

/*---------------------------------------------------------------------------*/
DbMatrix* DbHexaMatrixNew (int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  return (DbMatrixNew (DbTHexaNew (), rows, columns));
}

/*---------------------------------------------------------------------------*/
void DbHexaMatrixInit (DbMatrix* matrix, int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  DbMatrixInit (matrix, DbTHexaNew (), rows, columns);
}

/*---------------------------------------------------------------------------*/
unsigned int* DbHexaMatrixGetArray (DbMatrix* matrix)
/*---------------------------------------------------------------------------*/
{
  unsigned int* result;
  DbType type;

  if (matrix == NULL) return (NULL);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsHexa (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaMatrixGetArray>");
      return (NULL);
    }

  result = (unsigned int*) DbMatrixGetAddress (matrix, 0, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
unsigned int DbHexaMatrixGetElement (DbMatrix* matrix, int row, int column)
/*---------------------------------------------------------------------------*/
{
  unsigned int* result;
  DbType type;

  if (matrix == NULL) return (0);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsHexa (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaMatrixGetElement>");
      return (0);
    }

  result = (unsigned int*) DbMatrixGetAddress (matrix, row, column);

  return (*result);
}

/*---------------------------------------------------------------------------*/
void DbHexaMatrixSetElement (DbMatrix* matrix, 
                              int row, 
                              int column, 
                              unsigned int value)
/*---------------------------------------------------------------------------*/
{
  unsigned int* result;
  DbType type;

  if (matrix == NULL) return;

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsHexa (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbHexaMatrixSetElement>");
      return;
    }

  result = (unsigned int*) DbMatrixGetAddress (matrix, row, column);

  *result = value;
}

