#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Db.h>

/*---------------------------------------------------------------------------*/
DbReference* DbReferenceReferenceGetAddress (DbReference* reference)
/*---------------------------------------------------------------------------*/
{
  DbObject object;
  DbReference* result;

  object = DbReferenceGetObject (reference);

  if (!DbTypeIsReference (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbReferenceReferenceGetAddress>");
      return (NULL);
    }

  result = (DbReference*) DbReferenceObjectGetAddress (object);
  return (result);
}

/*---------------------------------------------------------------------------*/
DbReference* DbReferenceObjectLoad (char* name)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObject (name);
  if (DbGetError () != DbErrorOk) return (NULL);

  if (!DbTypeIsReference (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbReferenceObjectLoad>");
      return (NULL);
    }

  return ((DbReference*) DbReferenceObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
DbVector* DbReferenceVectorNew ()
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*  Warning : this function should change since the DbTReferenceNew requests */
/* a DbType argument. Currently (for bacward compatibility purposes) we      */
/* assume a "reference<>" with an unspecified type. It is therefore          */
/* recommended to directly use instead :                                     */
/*                                                                           */
/*   DbType my_type = ...;                                                   */
/*                                                                           */
/*   DbVectorNew (DbTReferenceNew (my_type));                                */
/*                                                                           */
/*  which actually corresponds to "reference <my_type>"                      */
/*                                                                           */
/*---------------------------------------------------------------------------*/
{
  return (DbVectorNew (DbTReferenceNew (NULL)));
}

/*---------------------------------------------------------------------------*/
void DbReferenceVectorInit (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  DbVectorInit (vector, DbTReferenceNew (NULL));
}

/*---------------------------------------------------------------------------*/
DbReference* DbReferenceVectorGetArray (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  DbReference* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsReference (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbReferenceVectorGetArray>");
      return (NULL);
    }

  result = (DbReference*) DbVectorGetAddress (vector, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
DbReference* DbReferenceVectorGetElement (DbVector* vector, int index)
/*---------------------------------------------------------------------------*/
{
  DbReference* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsReference (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbReferenceVectorGetElement>");
      return (NULL);
    }

  if (index >= DbVectorGetSize (vector))
    {
      DbVectorExtend (vector, index + 1);
    }

  result = (DbReference*) DbVectorGetAddress (vector, index);

  return (result);
}

/*---------------------------------------------------------------------------*/
DbMatrix* DbReferenceMatrixNew (int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  return (DbMatrixNew (DbTReferenceNew (NULL), rows, columns));
}

/*---------------------------------------------------------------------------*/
void DbReferenceMatrixInit (DbMatrix* matrix, int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  DbMatrixInit (matrix, DbTReferenceNew (NULL), rows, columns);
}

/*---------------------------------------------------------------------------*/
DbReference* DbReferenceMatrixGetArray (DbMatrix* matrix)
/*---------------------------------------------------------------------------*/
{
  DbReference* result;
  DbType type;

  if (matrix == NULL) return (NULL);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsReference (type)) return (NULL);
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbReferenceMatrixGetArray>");
      return (NULL);
    }

  result = (DbReference*) DbMatrixGetAddress (matrix, 0, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
DbReference* DbReferenceMatrixGetElement (DbMatrix* matrix, int row, int column)
/*---------------------------------------------------------------------------*/
{
  DbReference* result;
  DbType type;

  if (matrix == NULL) return (NULL);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsReference (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbReferenceMatrixGetElement>");
      return (NULL);
    }

  result = (DbReference*) DbMatrixGetAddress (matrix, row, column);

  return (result);
}
