#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <Db.h>

/*---------------------------------------------------------------------------*/
DbString* DbStringReferenceGetAddress (DbReference* reference)
/*---------------------------------------------------------------------------*/
{
  DbObject object;
  DbString* result;

  object = DbReferenceGetObject (reference);

  if (!DbTypeIsString (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringReferenceGetAddress>");
      return (NULL);
    }

  result = (DbString*) DbStringObjectGetAddress (object);
  return (result);
}

/*---------------------------------------------------------------------------*/
DbString* DbStringObjectLoad (char* name)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObject (name);
  if (DbGetError () != DbErrorOk) return (NULL);

  if (!DbTypeIsString (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringObjectLoad>");
      return (NULL);
    }

  return ((DbString*) DbStringObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
DbString* DbStringObjectLoadByTag (char* name, char* tag)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObjectByTag (name, tag);
  if (DbGetError () != DbErrorOk) return (NULL);

  if (!DbTypeIsString (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringObjectLoad>");
      return (NULL);
    }

  return ((DbString*) DbStringObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
DbString* DbStringObjectLoadByDate (char* name, char* date)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObjectByDate (name, date);
  if (DbGetError () != DbErrorOk) return (NULL);

  if (!DbTypeIsString (DbObjectGetType (object)))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringObjectLoad>");
      return (NULL);
    }

  return ((DbString*) DbStringObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
DbVector* DbStringVectorNew ()
/*---------------------------------------------------------------------------*/
{
  return (DbVectorNew (DbTStringNew ()));
}

/*---------------------------------------------------------------------------*/
void DbStringVectorInit (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  DbVectorInit (vector, DbTStringNew ());
}

/*---------------------------------------------------------------------------*/
DbString* DbStringVectorGetArray (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  DbString* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsString (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringVectorGetArray>");
      return (NULL);
    }

  result = (DbString*) DbVectorGetAddress (vector, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
DbString* DbStringVectorGetElement (DbVector* vector, int index)
/*---------------------------------------------------------------------------*/
{
  DbString* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsString (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringVectorGetElement>");
      return (NULL);
    }

  if (index >= DbVectorGetSize (vector))
    {
      DbVectorExtend (vector, index + 1);
    }

  result = (DbString*) DbVectorGetAddress (vector, index);

  return (result);
}

/*---------------------------------------------------------------------------*/
void DbStringVectorSetElement (DbVector* vector, 
                              int index, 
                              char* value)
/*---------------------------------------------------------------------------*/
{
  DbString* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIsString (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringVectorSetElement>");
      return;
    }

  if (index >= DbVectorGetSize (vector))
    {
      DbVectorExtend (vector, index + 1);
    }

  result = (DbString*) DbVectorGetAddress (vector, index);

  DbStringSetContents (result, value);

}

/*---------------------------------------------------------------------------*/
DbMatrix* DbStringMatrixNew (int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  return (DbMatrixNew (DbTStringNew (), rows, columns));
}

/*---------------------------------------------------------------------------*/
void DbStringMatrixInit (DbMatrix* matrix, int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  DbMatrixInit (matrix, DbTStringNew (), rows, columns);
}

/*---------------------------------------------------------------------------*/
DbString* DbStringMatrixGetArray (DbMatrix* matrix)
/*---------------------------------------------------------------------------*/
{
  DbString* result;
  DbType type;

  if (matrix == NULL) return (NULL);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsString (type)) return (NULL);
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringMatrixGetArray>");
      return (NULL);
    }

  result = (DbString*) DbMatrixGetAddress (matrix, 0, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
DbString* DbStringMatrixGetElement (DbMatrix* matrix, int row, int column)
/*---------------------------------------------------------------------------*/
{
  DbString* result;
  DbType type;

  if (matrix == NULL) return (NULL);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsString (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringMatrixGetElement>");
      return (NULL);
    }

  result = (DbString*) DbMatrixGetAddress (matrix, row, column);

  return (result);
}

/*---------------------------------------------------------------------------*/
void DbStringMatrixSetElement (DbMatrix* matrix, 
                              int row, 
                              int column, 
                              char* value)
/*---------------------------------------------------------------------------*/
{
  DbString* result;
  DbType type;

  if (matrix == NULL) return;

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIsString (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "DbStringMatrixSetElement>");
      return;
    }

  result = (DbString*) DbMatrixGetAddress (matrix, row, column);

  DbStringSetContents (result, value);
}
