/*---------------------------------------------------------------------------*/
/*        Copyright (c) 1996 LAL Orsay, UPS-IN2P3-CNRS (France).             */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/* 1. Redistributions of source code must retain the above copyright         */
/*    notice, this list of conditions and the following disclaimer.          */
/* 2. Redistributions in binary form must reproduce the above copyright      */
/*    notice, this list of conditions and the following disclaimer in the    */
/*    documentation and/or other materials provided with the distribution.   */
/* 3. All advertising materials mentioning features or use of this software  */
/*    must display the following acknowledgement:                            */
/*      This product includes software developed by the Computer Application */
/*      Development Group at LAL Orsay (Laboratoire de l'Accelerateur        */
/*      Linaire - UPS-IN2P3-CNRS).                                           */
/* 4. Neither the name of the Institute nor of the Laboratory may be used    */
/*    to endorse or promote products derived from this software without      */
/*    specific prior written permission.                                     */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE LAL AND CONTRIBUTORS ``AS IS'' AND       */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE     */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR        */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE LAL OR CONTRIBUTORS BE      */
/* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR       */
/* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF      */
/* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS  */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN   */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)   */
/* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF    */
/* THE POSSIBILITY OF SUCH DAMAGE.                                           */
/*---------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <CSet.h>
#include <DbHandle.h>

typedef struct _DbHandleRec
{
  void* reference;
} DbHandleRec;

/*---------------------------------------------------------------------------*/
static GCClass DbHandleGCClass = NULL;

static CLList Objects = NULL;
static CLList Types   = NULL;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
static DbHandle DbHandleNew ();
static void DbHandleDelete (DbHandle handle);
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
static DbHandle DbHandleNew ()
/*---------------------------------------------------------------------------*/
{
  DbHandle handle = NULL;

  if (DbHandleGCClass == NULL)
    {
      GCReferenceAssign (DbHandleGCClass,
                         GCClassNew ("DbHandle", 
                                     (GCDeleteMethod) DbHandleDelete));
    }

  handle = (DbHandle) malloc (sizeof (DbHandleRec));
  handle->reference = NULL;

  GCObjectNew (handle, DbHandleGCClass);

  return (handle);
}

/*---------------------------------------------------------------------------*/
static void DbHandleDelete (DbHandle handle)
/*---------------------------------------------------------------------------*/
{
  if (handle == NULL) return;

  if (handle->reference != NULL)
    {
      GCReferenceClear (handle->reference);
    }

  free (handle);
}

/*---------------------------------------------------------------------------*/
void DbHandleCleanup ()
/*---------------------------------------------------------------------------*/
{
  if (DbHandleGCClass != NULL)
    {
      CLListErase (Objects);
      CLListErase (Types);

      GCReferenceClear (Objects);
      GCReferenceClear (Types);

      GCReferenceClear (DbHandleGCClass);
    }
}

/*---------------------------------------------------------------------------*/
void DbHandleClear (DbHandle handle)
/*---------------------------------------------------------------------------*/
{
  if (handle == NULL) return;

  GCReferenceClear (handle->reference);
}

/*---------------------------------------------------------------------------*/
void DbHandleSet (DbHandle handle, void* reference)
/*---------------------------------------------------------------------------*/
{
  if (handle == NULL) return;

  GCReferenceAssignIn (handle->reference, handle, reference);
}

/*---------------------------------------------------------------------------*/
void* DbHandleGet (DbHandle handle)
/*---------------------------------------------------------------------------*/
{
  if (handle == NULL) return (NULL);

  return (handle->reference);
}

/*---------------------------------------------------------------------------*/
DbHandle DbObjectHandleNew ()
/*---------------------------------------------------------------------------*/
{
  DbHandle handle;

  if (Objects == NULL)
    {
      GCReferenceAssign (Objects, CLListNew ());
    }

  handle = DbHandleNew ();

  CLListAdd (Objects, handle);

  return (handle);
}

/*---------------------------------------------------------------------------*/
void DbObjectHandleRemove (DbHandle handle)
/*---------------------------------------------------------------------------*/
{
  if (handle == NULL) return;
  if (Objects == NULL) return;

  DbHandleClear (handle);
  CLListRemove (Objects, handle);
}

/*---------------------------------------------------------------------------*/
DbHandle DbTypeHandleNew ()
/*---------------------------------------------------------------------------*/
{
  DbHandle handle;

  if (Types == NULL)
    {
      GCReferenceAssign (Types, CLListNew ());
    }

  handle = DbHandleNew ();

  CLListAdd (Types, handle);

  return (handle);
}

/*---------------------------------------------------------------------------*/
void DbTypeHandleRemove (DbHandle handle)
/*---------------------------------------------------------------------------*/
{
  if (handle == NULL) return;
  if (Types == NULL) return;

  DbHandleClear (handle);
  CLListRemove (Types, handle);
}

