/*---------------------------------------------------------------------------*/
/*        Copyright (c) 1996 LAL Orsay, UPS-IN2P3-CNRS (France).             */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/* 1. Redistributions of source code must retain the above copyright         */
/*    notice, this list of conditions and the following disclaimer.          */
/* 2. Redistributions in binary form must reproduce the above copyright      */
/*    notice, this list of conditions and the following disclaimer in the    */
/*    documentation and/or other materials provided with the distribution.   */
/* 3. All advertising materials mentioning features or use of this software  */
/*    must display the following acknowledgement:                            */
/*      This product includes software developed by the Computer Application */
/*      Development Group at LAL Orsay (Laboratoire de l'Accelerateur        */
/*      Linaire - UPS-IN2P3-CNRS).                                           */
/* 4. Neither the name of the Institute nor of the Laboratory may be used    */
/*    to endorse or promote products derived from this software without      */
/*    specific prior written permission.                                     */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE LAL AND CONTRIBUTORS ``AS IS'' AND       */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE     */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR        */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE LAL OR CONTRIBUTORS BE      */
/* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR       */
/* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF      */
/* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS  */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN   */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)   */
/* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF    */
/* THE POSSIBILITY OF SUCH DAMAGE.                                           */
/*---------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <DbObject.h>

/*---------------------------------------------------------------------------*/
static int StringCount         = 0;
static int StringContentsCount = 0;
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
DbString* DbStringNew ()
/*---------------------------------------------------------------------------*/
{
  DbString* object = NULL;

  object = (DbString*) malloc (sizeof (DbString));
  StringCount++;

  DbStringInit (object);

  return (object);
}

/*---------------------------------------------------------------------------*/
void DbStringDelete (DbString* object)
/*---------------------------------------------------------------------------*/
{
  DbStringErase (object);

  free (object);
  StringCount--;
}

/*---------------------------------------------------------------------------*/
void DbStringInit (DbString* object)
/*---------------------------------------------------------------------------*/
{
  if (object == NULL) return;

  object->contents = NULL;
}

/*---------------------------------------------------------------------------*/
void DbStringErase (DbString* object)
/*---------------------------------------------------------------------------*/
{
  if (object == NULL) return;

  if (object->contents != NULL)
    {
      free (object->contents);
      StringContentsCount--;
      object->contents = NULL;
    }
}

/*---------------------------------------------------------------------------*/
void DbStringSetContents (DbString* object, char* text)
/*---------------------------------------------------------------------------*/
{
  if (object == NULL) return;

  if (text == NULL)
    {
      if (object->contents != NULL)
        {
          free (object->contents);
          StringContentsCount--;
          object->contents = NULL;
        }
      return;
    }

  if (object->contents == NULL)
    {
      object->contents = (char*) malloc (strlen (text) + 1);
      StringContentsCount++;
    }
  else
    {
      object->contents = (char*) realloc (
          object->contents, 
          strlen (text) + 1);
    }

  strcpy (object->contents, text);
}

/*---------------------------------------------------------------------------*/
char* DbStringGetContents (DbString* object)
/*---------------------------------------------------------------------------*/
{
  if (object == NULL) return (NULL);
  return (object->contents);
}

/*---------------------------------------------------------------------------*/
void DbStringAppend (DbString* object, char* text)
/*---------------------------------------------------------------------------*/
{
  if (object == NULL) return;

  if (text == NULL) return;

  if (object->contents == NULL)
    {
      object->contents = (char*) malloc (strlen (text) + 1);
      object->contents[0] = 0;

      StringContentsCount++;
    }
  else
    {
      object->contents = (char*) realloc (
          object->contents, 
          strlen (object->contents) + strlen (text) + 1);
    }

  strcat (object->contents, text);
}

/*---------------------------------------------------------------------------*/
void DbStringPrepend (DbString* object, char* text)
/*---------------------------------------------------------------------------*/
{
  if (object == NULL) return;

  if (text == NULL) return;

  if (object->contents == NULL)
    {
      object->contents = (char*) malloc (strlen (text) + 1);

      strcpy (object->contents, text);

      StringContentsCount++;
    }
  else
    {
      char* src;
      char* dest;
      int original_length = strlen (object->contents);
      int new_length = strlen (text);

      object->contents = (char*) realloc (
          object->contents, 
          original_length + new_length + 1);

      src = object->contents + original_length - 1;
      dest = object->contents + original_length + new_length - 1;

      while (original_length > 0)
        {
          *dest = *src;

          dest--;
          src--;
          original_length--;
        }

      strncpy (object->contents, text, new_length);
    }
}

/*---------------------------------------------------------------------------*/
int DbStringRemove (DbString* object, char* text)
/*---------------------------------------------------------------------------*/
{
  char* found;
  char* remaining;
  int position;
  int length;

  if (object == NULL) return (0);
  if (text == NULL) return (0);
  if (object->contents == NULL) return (0);

  found = strstr (object->contents, text);
  if (found == NULL) return (0);

  length = strlen (text);
  remaining = found + length;

  while (length > 0)
    {
      *found = *remaining;

      found++;
      remaining++;
      length--;
    }
  *found = 0;

  length = strlen (text);

  object->contents = (char*) realloc (object->contents, strlen (object->contents) + 1);

  return (1);
}

/*---------------------------------------------------------------------------*/
int DbStringReplace (DbString* object, char* original_text, char* new_text)
/*---------------------------------------------------------------------------*/
{
  char* found;
  char* remaining;
  int position;
  int removed_length;
  int old_length;
  int new_length;
  char* dest;
  char* old_contents;
  char* src;

  if (object == NULL) return (0);
  if (original_text == NULL) return (0);
  if (object->contents == NULL) return (0);

  found = strstr (object->contents, original_text);
  if (found == NULL) return (0);

    /*
      xxxxxxxxxxxooooooxxxxxxxxx
                 f     r

      o : original
      r : remaining

      */

  removed_length = strlen (original_text);
  remaining = found + removed_length;

  new_length = strlen (object->contents) - removed_length;
  if (new_text != NULL) new_length += strlen (new_text);

  old_contents = object->contents;
  src = old_contents;

  object->contents = (char*) malloc (new_length + 1);
  dest = object->contents;

  dest[0] = 0;
  position = 0;

  if (found > src)
    {
        /*
          The beginning of the old text is kept
          */
      position = found - src;
      strncpy (dest, src, position);
      dest += position;
      dest[0] = 0;
    }

  if ((new_text != NULL) &&
      (strlen (new_text) > 0))
    {
        /*
          replacement is not empty
          */
      strncpy (dest, new_text, strlen (new_text));
      dest += strlen (new_text);
      dest[0] = 0;
    }

    /*
      Append the remaining
      */

  strcat (dest, remaining);

  return (1);
}

/*---------------------------------------------------------------------------*/
void DbStringRemoveAll (DbString* object, char* text)
/*---------------------------------------------------------------------------*/
{
  while (DbStringRemove (object, text));
}

/*---------------------------------------------------------------------------*/
void DbStringReplaceAll (DbString* object, char* original_text, char* new_text)
/*---------------------------------------------------------------------------*/
{
  while (DbStringReplace (object, original_text, new_text));
}
