
static int DbTypeIs<T> (DbType type);
static <T>* Db<T>ObjectGetAddress (DbObject object);
static <T>* Db<T>ReferenceGetAddress (DbReference* reference);
static <T>* Db<T>ObjectLoad (char* name);
static <T>* Db<T>ObjectLoadByTag (char* name, char* tag);
static <T>* Db<T>ObjectLoadByDate (char* name, char* date);
static DbVector* Db<T>VectorNew ();
static void Db<T>VectorInit (DbVector* vector);
static <T>* Db<T>VectorGetArray (DbVector* vector);
static <T>* Db<T>VectorGetElement (DbVector* vector, int index);
static DbMatrix* Db<T>MatrixNew (int rows, int columns);
static void Db<T>MatrixInit (DbMatrix* matrix, int rows, int columns);
static <T>* Db<T>MatrixGetArray (DbMatrix* matrix);
static <T>* Db<T>MatrixGetElement (DbMatrix* matrix, int row, int column);

/*---------------------------------------------------------------------------*/
static int DbTypeIs<T> (DbType type)
/*---------------------------------------------------------------------------*/
{
  if (type == NULL) return (0);
  if (!DbTypeIsStructure (type)) return (0);
  return (!strcmp (DbTypeName (type), "<T>"));
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>ObjectGetAddress (DbObject object)
/*---------------------------------------------------------------------------*/
{
  <T>* result;

  if (object == NULL) return (NULL);

  if (!DbTypeIs<T> (DbObjectGetType (object))) 
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "Db<T>ObjectGetAddress>");
      return (NULL);
    }

  result = (<T>*) DbStructureObjectGetAddress (object);
  return (result);
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>ReferenceGetAddress (DbReference* reference)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbReferenceGetObject (reference);
  return (Db<T>ObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>ObjectLoad (char* name)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObject (name);
  if (DbGetError () != DbErrorOk) return (NULL);
  return (Db<T>ObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>ObjectLoadByTag (char* name, char* tag)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObjectByTag (name, tag);
  if (DbGetError () != DbErrorOk) return (NULL);
  return (Db<T>ObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>ObjectLoadByDate (char* name, char* date)
/*---------------------------------------------------------------------------*/
{
  DbObject object;

  object = DbLoadObjectByDate (name, date);
  if (DbGetError () != DbErrorOk) return (NULL);
  return (Db<T>ObjectGetAddress (object));
}

/*---------------------------------------------------------------------------*/
static DbVector* Db<T>VectorNew ()
/*---------------------------------------------------------------------------*/
{
  DbType type;

  type = DbTStructureFind ("<T>");
  if (type == NULL)
    {
      type = DbLoadType ("<T>");
    }
  if (type == NULL) return (NULL);

  return (DbVectorNew (type));
}

/*---------------------------------------------------------------------------*/
static void Db<T>VectorInit (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  DbType type;

  type = DbTStructureFind ("<T>");
  if (type == NULL)
    {
      type = DbLoadType ("<T>");
    }
  if (type == NULL) return;

  DbVectorInit (vector, type);
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>VectorGetArray (DbVector* vector)
/*---------------------------------------------------------------------------*/
{
  <T>* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIs<T> (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "Db<T>VectorGetArray>");
      return (NULL);
    }

  result = (<T>*) DbVectorGetAddress (vector, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>VectorGetElement (DbVector* vector, int index)
/*---------------------------------------------------------------------------*/
{
  <T>* result;
  DbType type;

  type = DbVectorGetElementType (vector);
  if (!DbTypeIs<T> (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "Db<T>VectorGetElement>");
      return (NULL);
    }

  if (index >= DbVectorGetSize (vector))
    {
      DbVectorExtend (vector, index + 1);
    }

  result = (<T>*) DbVectorGetAddress (vector, index);

  return (result);
}

/*---------------------------------------------------------------------------*/
static DbMatrix* Db<T>MatrixNew (int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  DbType type;

  type = DbTStructureFind ("<T>");
  if (type == NULL)
    {
      type = DbLoadType ("<T>");
    }
  if (type == NULL) return (NULL);

  return (DbMatrixNew (type, rows, columns));
}

/*---------------------------------------------------------------------------*/
static void Db<T>MatrixInit (DbMatrix* matrix, int rows, int columns)
/*---------------------------------------------------------------------------*/
{
  DbType type;

  type = DbTStructureFind ("<T>");
  if (type == NULL)
    {
      type = DbLoadType ("<T>");
    }
  if (type == NULL) return;

  DbMatrixInit (matrix, type, rows, columns);
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>MatrixGetArray (DbMatrix* matrix)
/*---------------------------------------------------------------------------*/
{
  <T>* result;
  DbType type;

  if (matrix == NULL) return (NULL);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIs<T> (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "Db<T>MatrixGetArray>");
      return (NULL);
    }

  result = (<T>*) DbMatrixGetAddress (matrix, 0, 0);

  return (result);
}

/*---------------------------------------------------------------------------*/
static <T>* Db<T>MatrixGetElement (DbMatrix* matrix, int row, int column)
/*---------------------------------------------------------------------------*/
{
  <T>* result;
  DbType type;

  if (matrix == NULL) return (NULL);

  type = DbMatrixGetElementType (matrix);
  if (!DbTypeIs<T> (type))
    {
      DbSetErrorInfo (DbErrorTypeMismatch, "Db<T>MatrixGetElement>");
      return (NULL);
    }

  result = (<T>*) DbMatrixGetAddress (matrix, row, column);

  return (result);
}

#endif
